/*
  ######################################################################
  # THIS IS THE STRIPPED VERSION OF THE ORIGINAL FILE                  #
  ######################################################################
  # This file was taken from libTiMidity 0.1.0 tar-ball distribution   #
  # Read the file "README.TXT" in this directory for more details      #
  ######################################################################
  # September 14, 2005 - Aloysius Indrayanto                           #
  # All comments (except the library copyright & license) are stripped #
  ######################################################################
  # See the VortexGE documentation for information about VortexGE      #
  ######################################################################
*/

/*
    TiMidity -- Experimental MIDI to WAVE converter
    Copyright (C) 1995 Tuukka Toivonen <toivonen[AT]clinet[DOT]fi>
    Copyright (C) 2004 Konstantin Korikov <lostclus[AT]ua[DOT]fm>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define OPEN_MODE "rb"

#include "timidity.h"
#include "timidity_internal.h"
#include "options.h"
#include "common.h"

static PathList *pathlist = NULL; 

FILE *open_file(char *name)
{
  FILE *fp;

  if (!name || !(*name))
    {
      DEBUG_MSG("Attempted to open nameless file.\n");
      return 0;
    }

  DEBUG_MSG("Trying to open %s\n", name);
  if ((fp = fopen(name, OPEN_MODE)))
    return fp;

  if (name[0] != PATH_SEP)
  {
    char current_filename[1024];
    PathList *plp = pathlist;
    int l;

    while (plp)  
      {
  *current_filename = 0;
  l = strlen(plp->path);
  if(l)
    {
      strcpy(current_filename, plp->path);
      if(current_filename[l - 1] != PATH_SEP)
      {
        current_filename[l] = PATH_SEP;
        current_filename[l + 1] = '\0';
      }
    }
  strcat(current_filename, name);
  DEBUG_MSG("Trying to open %s\n", current_filename);
  if ((fp = fopen(current_filename, OPEN_MODE)))
    return fp;
  plp = plp->next;
      }
  }
  
  DEBUG_MSG("Could not open %s\n", name);
  return 0;
}

void *safe_malloc(size_t count)
{
  void *p;

  p = malloc(count);
  if (p == NULL)
    DEBUG_MSG("Sorry. Couldn't malloc %d bytes.\n", count);

  return p;
}

void add_to_pathlist(char *s)
{
  PathList *plp = safe_malloc(sizeof(PathList));

  if (plp == NULL)
      return;

  plp->path = safe_malloc(strlen(s) + 1);
  if (plp->path == NULL)
  {
      free(plp);
      return;
  }

  strcpy(plp->path, s);
  plp->next = pathlist;
  pathlist = plp;
}

void free_pathlist(void)
{
    PathList *plp = pathlist;
    PathList *next;

    while (plp)
    {
  next = plp->next;
  free(plp->path);
  free(plp);
  plp = next;
    }
    pathlist = NULL;
}
