// ############################################################################################
//  VLGame-Entities.cc
//  A demo application for VortexGE (definition of VLGame's entity classes)
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _DEMO___VLGAME_ENTITIES_H_
#define _DEMO___VLGAME_ENTITIES_H_

// VortexGE headers
#include <VMain.h>


//
// This class could be moved to the VortexGE library
//
class CUnitSceneObjects {
  public:
    virtual ~CUnitSceneObjects() throw() {}

    virtual void UpdateScene() = 0;

    virtual bool PassableRegion(const TFloatXYZ &coord, const TFloatXYZ &radius) const = 0;

    virtual void RenderAllSolidObjects(const CMatrix3D &cameraMatrix) = 0;
    virtual void RenderAllTransparentObjects(const CMatrix3D &cameraMatrix) = 0;
};


//
// VLGame's keymap structure
//
struct VLGKeyMap {
  // Arrow keys
  bool up, down;
  bool left, right;

  inline void SetArrows(bool up_, bool down_, bool left_, bool right_)
  { up = up_; down = down_; left = left_; right = right_; }


  // Ctor So that all values in its inactive state
  inline VLGKeyMap()
  : up(false), down(false), left(false), right(false)
  {}
};


//
// VLGame's player class
//
class VLGPlayer : public CBasicCharacterSprite2D {
  public:
    VLGPlayer(const TInit_BasicCharSpr2D &init);

    void Initialize(const CAnimatedImage_MP &aimg, const TFloatXYZ &initPos);

    // Get the radius
    inline const TFloatXYZ &GetRadius() const
    { return(_rad); }
  
    // Get the current position
    inline const TFloatXYZ &GetPosition() const
    { return(_pos); }

    inline float GetXPos() const { return(_pos.x); }
    inline float GetYPos() const { return(_pos.y); }
    inline float GetZPos() const { return(_pos.z); }

    // Adjust the current position
    inline void AdjustXPos(float adj) { _pos.x += adj; }
    inline void AdjustYPos(float adj) { _pos.y += adj; }
    inline void AdjustZPos(float adj) { _pos.z += adj; }

    // Movement functions
    bool MoveLeft (const CUnitSceneObjects &usc, float amount = 4.0f);
    bool MoveRight(const CUnitSceneObjects &usc, float amount = 4.0f);
    bool MoveUp   (const CUnitSceneObjects &usc, float amount = 4.0f);
    bool MoveDown (const CUnitSceneObjects &usc, float amount = 4.0f);
    bool Move     (const CUnitSceneObjects &usc, bool up, bool down, bool left, bool right);

    // Camera matrix functions
           void             FCamUpdate();
    inline const CMatrix3D &FCamMatrix() const { return(_fCam); }

  private:
    // Constants
    static const float CameraYPos;
    static const float CameraZPos;

    TFloatXYZ _rad;           // Player's radius
    TFloatXYZ _pos;           // Player's position
    CMatrix3D _fCam;          // Follow-camera's matrix
    float     _fCamPos[3];    // Follow-camera's position
    float     _fCamTarget[3]; // Follow-camera's target
};


//
// VLGame's VLGUnitScene_StartRoom class
//
class VLGUnitScene_StartRoom : public CUnitSceneObjects {
  public:
    VLGUnitScene_StartRoom(CLuaManager        &luaManager,
                           CArchiveDataSource &mediaPackage,
                           CMediaPool      &mediaPool,
                           C3DEnv             &i3DEnv);
    virtual ~VLGUnitScene_StartRoom() throw();

    virtual void UpdateScene();

    virtual bool PassableRegion(const TFloatXYZ &coord, const TFloatXYZ &radius) const;

    virtual void RenderAllSolidObjects(const CMatrix3D &cameraMatrix);
    virtual void RenderAllTransparentObjects(const CMatrix3D &cameraMatrix);

    bool InDoorAccessArea(const TFloatXYZ &coord, const TFloatXYZ &radius) const;

    inline bool DoorIsOpen() const { return(_doorOpened); }
    inline void OpenDoor()         { _doorOrder =  1; }
    inline void CloseDoor()        { _doorOrder = -1; }

  private:
    CLuaManager            &_luaManager;
    CArchiveDataSource     &_mediaPackage;
    CMediaPool          &_mediaPool;
    C3DEnv                 &_i3DEnv;
    CBasicCharacterSprite2D _fireAnim;
    CPanel3DBuffer          _room;
    CPanel3DBuffer          _door;
    CPanel3DBuffer          _path;
    CPanel3DBuffer          _soil;
    bool                    _doorOpened;
    int32_t                 _doorOrder;
    int32_t                 _doorCnt;

    TFloatXYZ               _areaFloor[2];
    TFloatXYZ               _areaDoor[2];
    TFloatXYZ               _areaPath[2];
};

#endif
