/** \file PixelColorInterpolatorFractals.h */
// ############################################################################################
//  PixelColorInterpolatorFractals.h
//  Definition & implementation of various CPCIFractal derivative classes
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _2D_MANIPULATION___FRACTAL___PIXEL_COLOR_INTERPOLATOR_FRACTALS_H_
#define _2D_MANIPULATION___FRACTAL___PIXEL_COLOR_INTERPOLATOR_FRACTALS_H_

// VortexGE headers
#include "DefiniteAreaFractals.h"


namespace vge {

  //!
  //! \brief Fractal that will draw a target box
  //!
  //! \api experiment
  class VGE_EXPORT FractalTargetBox : public CPCIFractal {
    protected:
      // Parameter-control functions (all coordinates are in standard screen coordinates)
      virtual bool DrawCenterPixel() { return(false); }
      virtual void DrawSeed(int32_t n32X1, int32_t n32Y1, int32_t n32X2, int32_t n32Y2);
  };


  //!
  //! \brief Fractal that will try to imitate nebula
  //!
  //! \api experiment
  class VGE_EXPORT FractalNebula : public CPCIFractal {
    protected:
      // Parameter-control functions (all coordinates are in standard screen coordinates)
      virtual void    DrawSeed(int32_t n32X1, int32_t n32Y1, int32_t n32X2, int32_t n32Y2);
      virtual uint8_t RandomFactor(int32_t n32Distance);
  };


  //!
  //! \brief Fractal that will try to imitate coral
  //!
  //! \api experiment
  class VGE_EXPORT FractalCoral : public CPCIFractal {
    protected:
      // Parameter-control functions (all coordinates are in standard screen coordinates)
      virtual void    DrawSeed(int32_t n32X1, int32_t n32Y1, int32_t n32X2, int32_t n32Y2);
      virtual uint8_t RandomFactor(int32_t n32Distance);
  };

} // namespace vge
#endif
