/** \file NonDummySoundBufferBaseClass.h */
// ############################################################################################
//  NonDummySoundBufferBaseClass.h
//  Definition of CSoundBufferCImp class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _AUDIO_ENVIRONMENT___NON_DUMMY_SOUND_BUFFER_BASE_CLASS_H_
#define _AUDIO_ENVIRONMENT___NON_DUMMY_SOUND_BUFFER_BASE_CLASS_H_

// VortexGE headers
#include "NonDummyAudioEnvBaseClass.h"
#include "SoundBuffer.h"


namespace vge {

  //!
  //! \brief Non-dummy audio sound-buffer common class
  //!
  //! \api pretty stable
  class VGE_EXPORT CSoundBufferCImp : public CSoundBuffer {
    public:
      virtual ~CSoundBufferCImp() throw();
      // Playback functions
      virtual void Play(bool bLooping = false);
      virtual bool IsPlaying() const;
      virtual void Stop();
      // Sound buffer functions
      virtual void AdjustAmplitude(float fLeftGain, float fRightGain);
      virtual void ConvertMonoToStereo(bool bSilentRightChannel = false);
      virtual void ConvertStereoToMono();
      virtual void Convert16BitTo8Bit();
      virtual void SwapLRChannels();
      // I/O functions
      virtual void LoadRAW(const CDataSource &source, ERawFormat eFormat, int32_t n32Frequency);
      virtual void LoadWAV(const CDataSource &source);
      virtual void LoadVSW(const CDataSource &source);
      virtual void SaveVSW(const CDataSink &sink) const;

      // Friend classes
      friend class CAudioEnvCImp;

    protected:
      // Make sure user can't construct this class directly
      CSoundBufferCImp();

      // Sound buffer functions
      static void _ExtractFormat_(_TAudioFormat *ptFormat, ERawFormat eFormat);
      void _SwapEndian_();
      void _InvertSign_();
      void _Mono2Stereo_(bool bSilentRightChannel);
      void _Stereo2Mono_();
      void _16BitTo8Bit_();
      void _AssignCnvFunc_(bool bDifFreq);
      void _FinalizeLoad_();
      void _FreeBuffer_();
      // Audio environment access functions
      inline uint32_t _primaryBufferFrequency_()const
        { return(CAudioEnvCImp::_instAudioEnv->_tCD->tFormat.un32Frequency); }
      inline uint8_t _primaryBufferChannels_()const
        { return(CAudioEnvCImp::_instAudioEnv->_tCD->tFormat.un8Channels); }
      inline uint8_t _primaryBufferBits_()const
        { return(CAudioEnvCImp::_instAudioEnv->_tCD->tFormat.un8Bits); }

      // Data
      _TSBData      _tD; // The sound buffer data
      _TAudioFormat _tF; // The audio format
  
      // Pointer to the get-data function
      typedef void (*_XCnvFunc_)(_TPlayedSBData&, u_char*);
      _XCnvFunc_ _pfnGD_;

      // Called by sound mixing function to get the sound buffer data
      inline void _GetData_(_TPlayedSBData &tData, u_char *pucDstBuff)
      const { _pfnGD_(tData, pucDstBuff); }
  };

} // namespace vge
#endif
