// ############################################################################################
//  BasicEntities.cc
//  Implementation of various basic game entity classes
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGE headers
#include "../2DManipulation/AnimatedImageController.h"
#include "../2DManipulation/ImageClasses.h"
#include "BasicEntities.h"


namespace vge {

//
// CGenttyBase implementation
//
CGenttyBase::CGenttyBase(const char *entityName)
: _name(entityName ? entityName : "")
{
}

CGenttyBase::~CGenttyBase() throw()
{
}


//
// CGenttyBaseXSpace implementation
//
CGenttyBaseXSpace::CGenttyBaseXSpace(const char  *entityName,
                                     const char  *xspaceName,
                                     const char  *xspaceGroupName,
                                     CLuaManager *scriptManager)
: CGenttyBase(entityName),
  _xSpace(xspaceName ? xspaceName : entityName, xspaceGroupName)
{
  if(scriptManager) scriptManager->LXRegister(_xSpace);
}

CGenttyBaseXSpace::~CGenttyBaseXSpace() throw()
{
}


//
// CGenttyBaseEBC implementation
//
CGenttyBaseEBC::CGenttyBaseEBC (const char  *entityName,
                                const char  *exeName,
                                CLuaManager *scriptManager)
: CGenttyBase(entityName),
  _exe(exeName ? exeName : entityName)
{
  if(scriptManager) scriptManager->LERegister(_exe);
}

CGenttyBaseEBC::~CGenttyBaseEBC() throw()
{
}

void CGenttyBaseEBC::ExecuteScript(CGenttyBaseXSpace &xspaceEntity)
{
  xspaceEntity.iXSpace().Execute(_exe, NULL);
}


//
// CGenttyBaseXSpaceEBC implementation
//
CGenttyBaseXSpaceEBC::CGenttyBaseXSpaceEBC(const char  *entityName,
                                           const char  *xspaceName,
                                           const char  *xspaceGroupName,
                                           const char  *exeName,
                                           CLuaManager *scriptManager)
: CGenttyBase      (entityName),
  CGenttyBaseXSpace(entityName, xspaceName, xspaceGroupName, scriptManager),
  CGenttyBaseEBC   (entityName, exeName, scriptManager)
{
}

CGenttyBaseXSpaceEBC::~CGenttyBaseXSpaceEBC() throw()
{
}

void CGenttyBaseXSpaceEBC::ExecuteScript(CGenttyBaseXSpace &xspaceEntity)
{
  xspaceEntity.iXSpace().Execute(iEBC(), &iXSpace());
}


//
// CGenttyBaseImage implementation
//
CGenttyBaseImage::CGenttyBaseImage(const char      *entityName,
                                   const CImage_MP &image)
: CGenttyBase(entityName), _image(image)
{
}

CGenttyBaseImage::~CGenttyBaseImage() throw()
{
}


//
// CGenttyBaseAImage implementation
//
const CGenttyBaseAImage::CInit_AnimSet  CGenttyBaseAImage::_CEAS_;
const CGenttyBaseAImage::CInit_AnimSet *CGenttyBaseAImage::CreateEmptyAnimationSet = &CGenttyBaseAImage::_CEAS_;

CGenttyBaseAImage::CGenttyBaseAImage(const char              *entityName,
                                     const CAnimatedImage_MP &aimage,
                                     const CInit_AnimSet     *asInit)
: CGenttyBase(entityName), _aimage(aimage), _aictl(NULL)
{
  if(asInit) {
    _aictl = new CAImgControl;

    for(size_t i = 0; i < asInit->size(); ++i) {
      // Get the total frames count
      const CInit_AnimSetFrame &fi = (*asInit)[i];
      if(!fi.size()) continue;
      // Add the frames
      const uint32_t idx = _aictl->AddAnimationSet(fi.size());
      for(size_t j = 0; j < fi.size(); ++j) _aictl->AddASFrame(idx, fi[j]);
    }
  }
}

CGenttyBaseAImage::~CGenttyBaseAImage() throw()
{
  delete _aictl;
}

} // namespace vge
