// ############################################################################################
//  DialogBase.cc
//  Implementation of VortexGE base dialog controls
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGE headers
#include "DialogBase.h"


namespace vge {

// Default text to be used for measuring UTF-8 font height
const char *_DTM = "GgQq[]";


//
// The base of all dialog's control classes
//
_CControl::~_CControl() throw()
{
}

void _CControl::InitBaseData(CVideoEnv &vidEnv, const _CControl *pcsParent, ECtrlID eID, int32_t n32X, int32_t n32Y, uint32_t un32Width, uint32_t un32Height)
{
  // Set variables
  _tBD.eID       = eID;
  _tBD.pcsVidEnv = &vidEnv;
  _tBD.pcsParent = pcsParent;
  _tBD.bEnabled  = true;
  _tBD.un32W     = un32Width;
  _tBD.un32H     = un32Height;
  _tBD.n32X1     = n32X;
  _tBD.n32Y1     = n32Y;

  // Calculate n32X2 and n32Y2
  if(_tBD.pcsParent) {
    _tBD.n32X1 += _tBD.pcsParent->_tBD.n32X1;
    _tBD.n32Y1 += _tBD.pcsParent->_tBD.n32Y1;
  }
  _tBD.n32X2 = _tBD.n32X1 + _tBD.un32W - 1;
  _tBD.n32Y2 = _tBD.n32Y1 + _tBD.un32H - 1;
}

bool _CControl::MouseOver(int32_t n32MouseX, int32_t n32MouseY) const 
{
  if(!_tBD.bEnabled) return(false);
  if((n32MouseX >= _tBD.n32X1) && (n32MouseX <= _tBD.n32X2) &&
    (n32MouseY >= _tBD.n32Y1) && (n32MouseY <= _tBD.n32Y2))
    return(true);
  return(false);
}

bool _CControl::MouseOver(int32_t n32X1, int32_t n32Y1, int32_t n32X2, int32_t n32Y2, int32_t n32MouseX, int32_t n32MouseY) const 
{
  if(!_tBD.bEnabled) return(false);
  if((n32MouseX >= n32X1) && (n32MouseX <= n32X2) &&
    (n32MouseY >= n32Y1) && (n32MouseY <= n32Y2))
    return(true);
  return(false);
}


//
// The base for all control classes which store string
//
_CStringControl::_CStringControl()
{ 
  _tSD._len = 0; 
}

_CStringControl::~_CStringControl() throw()
{
}


//
// A class used for storing controls in a list
//
void CControlList::add(_CControl *control)
{
  _clist.push_back(control);
}

void CControlList::add(CControlList &clist)
{
  for(_CControlList::iterator it = clist._clist.begin(); it != clist._clist.end(); ++it)
    add(*it);
}

void CControlList::remove(CControlList &clist)
{
  for(_CControlList::iterator it = clist._clist.begin(); it != clist._clist.end(); ++it)
    remove(*it);
}

CControlList::iterator CControlList::begin()
{
  _CControlList::iterator it = _clist.begin();
  return(iterator(it));
}

CControlList::const_iterator CControlList::begin() const
{
  _CControlList::iterator it = const_cast<_CControlList&>(_clist).begin();
  return(const_iterator(it));
}

CControlList::iterator CControlList::end()
{
  _CControlList::iterator it = _clist.end();
  return(iterator(it));
}

CControlList::const_iterator CControlList::end() const
{
  _CControlList::iterator it = const_cast<_CControlList&>(_clist).end();
  return(const_iterator(it));
}

} // namespace vge
