// ############################################################################################
//  mcatedit-Main.cc
//  Implementation of mcatedit's utility application
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// mcatedit's headers
#include "mcatedit-MCatEdit.h"


//
// Main program
//
int main(int argc, char *argv[])
{
  // Minimum width and height
  const int minWidth  = 560;
  const int minHeight = 400;

  // Default parameters
  int             width    = 99999;
  int             height   = 99999;
  string          startDir = VGetCWD();
  CMCatEdit::Mode mode     = CMCatEdit::ModeNormal;

  // Parse parameters
  for(int i = 1; i < argc; ++i) {
    const size_t argl = strlen(argv[i]);

    if((argl > 8) && (strncmp(argv[i], "--width=", 8) == 0))
      width = atoi(argv[i] + 8);
    else if((argl > 9) && (strncmp(argv[i], "--height=", 9) == 0))
      height = atoi(argv[i] + 9);
    else if(((argl > 11) && strncmp(argv[i], "--startdir=", 11) == 0)) {
      startDir = argv[i] + 11;
      if(startDir[startDir.length() - 1] != '/') startDir += '/';
    }
    else if(strcmp(argv[i], "--new") == 0)
      mode = CMCatEdit::ModeNew;
    else if(strcmp(argv[i], "--open") == 0)
      mode = CMCatEdit::ModeOpen;
    else {
      VPrint("\n-------------------------------------------");
      VPrint("\nVortexGE Message Catalog Editor %s", CMCatEdit::TEXT_VERSION.c_str());
      VPrint("\nCopyright (C) %s Aloysius Indrayanto", CMCatEdit::TEXT_YEAR.c_str());
      VPrint("\n-------------------------------------------");
      VPrint("\nThis program is part of VortexGE and is licensed under the GNU LGPL.\n");
      VPrint("\n");
      VPrint("\nINVALID OPTION: %s\n", argv[i]);
      VPrint("\n");
      VPrint("\nSupported options:\n");
      VPrint("\n  --width=WINDOW_WIDTH");
      VPrint("\n      set the width of the window (minimum %d pixels); the default is to fill the whole desktop width.", minWidth);
      VPrint("\n");
      VPrint("\n  --height=WINDOW_HEIGHT");
      VPrint("\n      set the height of the window (minimum %d pixels); the default is to fill the whole desktop height.", minHeight);
      VPrint("\n");
      VPrint("\n  --startdir=START_DIR");
      VPrint("\n      set the startdirectory.");
      VPrint("\n");
      VPrint("\n  --new");
      VPrint("\n      create a new catalog in the start directory (or in the "
                     "current working directory if --startdir is not specified).");
      VPrint("\n");
      VPrint("\n  --open");
      VPrint("\n      open the catalog in the start directory (or in the current "
                     "working directory if --startdir is not specified).");
      VPrint("\n\n");
      return(1);
    }
  }

  // Execute the application
  CMCatEdit csMyApp(max(minWidth, width), max(minHeight, height), startDir, mode);
  return(csMyApp.DoMainLoop());
}
