// ############################################################################################
//  roomedit-DlgPages-Main-Common.cc
//  Implementation of roomedit's CPage_Main class (common)
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// roomedit's headers
#include "roomedit-BIcon.h"
#include "roomedit-DlgPages-Main.h"
#include "roomedit-RoomEdit.h"


//
// Our main dialog page
//
const uint32_t CPage_Main::DIRTY_MAX;

const uint32_t CPage_Main::OBJECT_TAB_PLANE;
const uint32_t CPage_Main::OBJECT_TAB_MAX;


CPage_Main::~CPage_Main() throw()
{
}

CPage_Main::CPage_Main(CRoomEdit &appClass, bool lowRes3D_)
: CDialogPage     (appClass, &appClass),
  _app            (appClass),
  _activeObjectTab(OBJECT_TAB_PLANE),
  _dirty          (10),
  lowRes3D        (lowRes3D_)
{
  _iVideoEnv->ClearMainPixmap(_iVideoEnv->CreateColor(0, 0, 0));
  _iVideoEnv->UseDefaultBigFont();
  _iVideoEnv->SetForegroundColor(_iVideoEnv->CreateColor(128, 164, 192));
  _iVideoEnv->DrawText(0, 150, "Initializing system, please wait ...", ETA_CenterScreen);
  _iVideoEnv->UseDefaultSmallFont();
  _iVideoEnv->UpdateDisplay();
  VInfo(_NFL_, "Initializing system, please wait ...");

  _vStrg.Initialize(_iVideoEnv, 1);
}


// Management functions
void CPage_Main::ResetAllControls()
{
  // Operations section - camera mode
  rdoCamFollow.SetValue(true);
  rdoCamFree.SetValue(false);
  chkCamFreePos.SetValue(true);
  chkCamFreeTrg.SetValue(true);

  // Operations section - movement mode
  rdoMovCurObj.SetValue(false);
  rdoMovCurOnly.SetValue(true);
  rdoMovCam.SetValue(false);
}

void CPage_Main::UpdateAllControlsEnableStates()
{
  const bool isOpen = _roomBuilder.IsOpen();

  // Menu bar section
  btnMNSave.SetEnable(isOpen);
  btnMNSaveAs.SetEnable(isOpen);
  // Object list section
  lstObjects.SetEnable(isOpen);
  btnEditObject.SetEnable(isOpen);
  btnRenObject.SetEnable(isOpen);
  btnAlignObject.SetEnable(isOpen);
  btnCloneObject.SetEnable(isOpen);
  btnDelObject.SetEnable(isOpen);
  // Operations section - camera mode
  rdoCamFollow.SetEnable(isOpen);
  rdoCamFree.SetEnable(isOpen);
  chkCamFreePos.SetEnable(isOpen);
  chkCamFreeTrg.SetEnable(isOpen);
  // Operations section - movement mode
  rdoMovCurObj.SetEnable(isOpen);
  rdoMovCurOnly.SetEnable(isOpen);
  rdoMovCam.SetEnable(isOpen);
  // Operations section - objects (main)
  btnGrid.SetEnable(isOpen);
  // Operations section - objects (plane)
  btnNewPlane.SetEnable(isOpen);
  // Operations section - objects (camera)
  for(size_t i = 0; i < elemsize(btnResetCamPos); ++ i) btnResetCamPos[i].SetEnable(isOpen);
}

void CPage_Main::UpdateInfoText()
{
  if(!_roomBuilder.IsOpen()) {
    lblGridSz[2].SetString("-");
    lblCurPos[2].SetString("-");
    lblCamPos[2].SetString("-");
    lblCamTrg.SetString("-");
    return;
  }

  const TFloatXYZ &gridSize = _roomBuilder.GetRoomGridSize();
  const TFloatXYZ &curPos   = _loc.curPos;
  const TFloatXYZ &camPos   = _loc.camPos;
  const TFloatXYZ &camTrg   = _loc.camTrg;

  char buff[1024];
  snprintf(buff, sizeof(buff),
           "%+08.2f, %+08.2f, %+08.2f",
           gridSize.x, gridSize.y, gridSize.z); lblGridSz[2].SetString(buff);
  snprintf(buff, sizeof(buff),
           "%+08.2f, %+08.2f, %+08.2f",
           curPos.x, curPos.y, curPos.z); lblCurPos[2].SetString(buff);
  snprintf(buff, sizeof(buff),
           "%+08.2f, %+08.2f, %+08.2f",
           camPos.x, camPos.y, camPos.z); lblCamPos[2].SetString(buff);
  snprintf(buff, sizeof(buff),
           "%+08.2f, %+08.2f, %+08.2f",
           camTrg.x, camTrg.y, camTrg.z); lblCamTrg.SetString(buff);

  pnlCamMovInfo.DrawControl();
  for(size_t i = 0; i < 3; ++i) {
    lblGridSz[i].DrawControl();
    lblCurPos[i].DrawControl();
    lblCamPos[i].DrawControl();
  }
  lblCamTrg.DrawControl();
}

static const TFloatXYZ _CalcMF_(const CPage_Main::_TKeyMap &keyMap,
                                const TFloatXYZ            &gridSize,
                                const TFloatXYZ            &camPos,
                                const TFloatXYZ            &camTrg)
{
  // Movement on the Y-axis
  const float yf = gridSize.y / 10.0f;
  if(keyMap.home) return(TFloatXYZ(0, -yf, 0));
  if(keyMap.pgup) return(TFloatXYZ(0,  yf, 0));

  // Movement on the XZ-plane
  if(keyMap.up || keyMap.down || keyMap.left || keyMap.right) {
    // Create XZ-vector from the camera's XZ-target to the camera's XZ-position
    const float dx = camPos.x - camTrg.x;
    const float dz = camPos.z - camTrg.z;
    const float ax = fabs(dx);
    const float az = fabs(dz);
    // Our scaled degree
    const int32_t a360 = xprv::_DEGREECOUNT_;
    const int32_t a270 = xprv::_DEGREECOUNT_ * 3 / 4;
    const int32_t a180 = xprv::_DEGREECOUNT_ * 2 / 4;
    const int32_t a90  = xprv::_DEGREECOUNT_ * 1 / 4;
    // Calculate the angle formed by the positive-X-axis in the XZ-plane and
    // the XZ-vector
    int32_t angle;
         if(ax == 0.0f) angle = (dz >= 0) ? a90 : a270;
    else if(az == 0.0f) angle = (dx >= 0) ?   0 : a180;
    else {
      angle = int32_t(a180 / M_PI * atanf(az / ax));
           if(dx>0 && dz>0) {}                    // Quadrant I
      else if(dx>0 && dz<0) angle = a360 - angle; // Quadrant IV
      else if(        dz>0) angle = a180 - angle; // Quadrant II
      else                  angle = a180 + angle; // Quadrant III
    }
    // Calculate the movement factor
    const float sinA = SIN(angle);
    const float cosA = COS(angle);
    const float ff   = min(gridSize.y, gridSize.z) / 10.0f;
         if(keyMap.up)    return(TFloatXYZ(-ff * cosA, 0, -ff * sinA));
    else if(keyMap.down)  return(TFloatXYZ( ff * cosA, 0,  ff * sinA));
    else if(keyMap.left)  return(TFloatXYZ( ff * sinA, 0, -ff * cosA));
    else if(keyMap.right) return(TFloatXYZ(-ff * sinA, 0,  ff * cosA));
  }

  // Other
  return(TFloatXYZ(0, 0, 0));
}

static const TFloatXYZ _AlignMF_(const TFloatXYZ &mf,
                                 const TFloatXYZ &gridSize)
{
  const float xf  = gridSize.x / 10.0f;
  const float yf  = gridSize.y / 10.0f;
  const float zf  = gridSize.z / 10.0f;
  const float xf2 = xf / 2.0f;
  const float yf2 = yf / 2.0f;
  const float zf2 = zf / 2.0f;

  // Return the aligned movement factor
  return( TFloatXYZ( (fabs(mf.x) < xf2) ? 0.0f : ((mf.x >= 0.0f) ? xf : -xf),
                     (fabs(mf.y) < yf2) ? 0.0f : ((mf.y >= 0.0f) ? yf : -yf),
                     (fabs(mf.z) < zf2) ? 0.0f : ((mf.z >= 0.0f) ? zf : -zf) ) );
}

void CPage_Main::HandleMovements()
{
  // For single step movement
  static bool done = false;
  static bool step = false;
  step |= _keyMap.ctrl;
  if(_keyMap.up || _keyMap.down || _keyMap.left || _keyMap.right || _keyMap.home || _keyMap.pgup) {
    if(step && done) return;
    done = true;
  }
  else {
    step = false;
    done = false;
    return;
  }

  // For movement rate control
  static uint64_t oldTicks = VGetSystemTicks();
         uint64_t curTicks = VGetSystemTicks();
  if(curTicks-oldTicks <  TICKS_PER_SEC/25) return;
  oldTicks = curTicks;

  // For convenience
  const float &xg = _roomBuilder.GetRoomGridSize().x;
  const float &yg = _roomBuilder.GetRoomGridSize().y;
  const float &zg = _roomBuilder.GetRoomGridSize().z;
  const float  mg = min(min(xg, yg), zg) / 10.0f;

  // Calculate the movement factor
  const TFloatXYZ &mf = _CalcMF_(_keyMap, _roomBuilder.GetRoomGridSize(),
                                 _loc.camPos, _loc.camTrg);
  if(fabs(mf.x)+fabs(mf.y)+fabs(mf.z) < mg) return;

  // Move cursor (and object) ?
  if(rdoMovCurObj.GetValue() || rdoMovCurOnly.GetValue()) {
    const TFloatXYZ &af = _AlignMF_(mf, _roomBuilder.GetRoomGridSize());
    // Move the cursor
    _loc.curPos += af;
    // Shall the selected object be moved too ?
    if(rdoMovCurObj.GetValue()) {
      if(_activeObjectTab == OBJECT_TAB_PLANE) _onMove_Plane(af);
    }
    MarkAsDirty();
  }
  // Move camera ?
  else if(rdoMovCam.GetValue()) {
    // Follow-camera ?
    if(rdoCamFollow.GetValue()) {
      _loc.lCamOffset += mf;
      MarkAsDirty();
    }
    // Free-camera ?
    else if(rdoCamFree.GetValue()) {
      // Move the position ?
      if(chkCamFreePos.GetValue()) {
        _loc.fCamPos += mf;
         MarkAsDirty();
      }
      // Move the target ?
      if(chkCamFreeTrg.GetValue()) {
        _loc.fCamTrg += mf;
        MarkAsDirty();
      }
    }
  }
}


// Rendering functions
void CPage_Main::MarkAsDirty()
{
  if(_roomBuilder.IsOpen()) _dirty = DIRTY_MAX;
}

void CPage_Main::UpdateCameraPos()
{
  // Follow-camera ?
  if(rdoCamFollow.GetValue()) {
    _loc.camTrg = _loc.curPos;
    _loc.camPos = _loc.camTrg + _loc.lCamOffset;
  }
  // Free-camera ?
  else if(rdoCamFree.GetValue()){
    _loc.camTrg = _loc.fCamTrg;
    _loc.camPos = _loc.fCamPos;
  }

  // Calculate the camera matrix
  const float pos[3] = { _loc.camPos.x, _loc.camPos.y, _loc.camPos.z };
  const float trg[3] = { _loc.camTrg.x, _loc.camTrg.y, _loc.camTrg.z };
  _loc.matCam.SetToLookAtMatrix(pos, trg);
}

void CPage_Main::RenderRoom(bool updateFrameInfo)
{
  // Update the frame and the information text if needed
  if(updateFrameInfo) {
    UpdateCameraPos();
    UpdateInfoText();
  }

  // Clear Z-buffer
  _i3DEnv->ClearRenderBuffer();
  if(!_roomBuilder.IsOpen()) return;
  _i3DEnv->ClearZBuffer();

  // Render the room
  const char *planeName = lstObjects.GetSelectedItem();
  _roomBuilder.RenderAll(_loc.matCam, planeName ? planeName : "");

  // Matrix to be used to render the editor's informative objects
  static CMatrix3D lm;

  // Render the camera target
  lm.SetToTranslationMatrix(_loc.camTrg.x, _loc.camTrg.y, _loc.camTrg.z);
  lm.MultMatrixCCN(_loc.matCam);
  _camTarget.CFRender(lm);

  // Render the cursor and grid
  lm.SetToTranslationMatrix(_loc.curPos.x, _loc.curPos.y, _loc.curPos.z);
  lm.MultMatrixCCN(_loc.matCam);
  _cursor.CFRender(lm);
  _grid.TNTransRender(lm);
}

void CPage_Main::SmartUpdateAll()
{
  if(!_dirty) return;

  _dirty--;
  RenderRoom(true);

  while(--_dirty) RenderRoom(false);
  
  CustomDraw_OnRedrawAllControls();
  _iVideoEnv->UpdateDisplay();
}


// Other member functions
void CPage_Main::InitializeAllControls()
{
  _iVideoEnv->UseDefaultWaitCursor();

  // Register keyboard shortcuts
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscCamFollow ), 'f');
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscCamFree   ), 'r');
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscCamMovPos ), 'p');
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscCamMovTrg ), 't');
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscMovCurObj ), 'j');
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscMovCurOnly), 'c');
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscMovCam    ), 'm');
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscMovCam2F  ), 'f', false, false, true);
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscMovCam2L  ), 'l', false, false, true);
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscMovCam2R  ), 'r', false, false, true);
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscMovCam2B  ), 'b', false, false, true);
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscEditObj   ), 'e', true);
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscRenObj    ), 'r', true);
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscAlignObj  ), 'a', true);
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscCloneObj  ), 'c', true);
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscDelObj    ), VKS_DEL);
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscDelObj    ), VKS_DEL);
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscNewPlane  ), 'p', true, false, true);
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscGrid      ), 'g', true);
  RegisterKeyboardShortcut(CDP_SLOT(CPage_Main, _onKscHelp      ), VKS_F10);

  // Main section
  const uint32_t MENU_STATUS_BAR_HEIGHT = 25;
  size_t idxMain = 0;
  InitControl(hlyMain, NULL, 0, 0);
      hlyMain.AddBlocksEx(3, MENU_STATUS_BAR_HEIGHT, 0, MENU_STATUS_BAR_HEIGHT);

  // Menu bar section
  size_t idxMenu = 0;
  InitControl(vlyMenu, hlyMain, idxMain, 0, 0);
      vlyMenu.AddBlocks(5);
      idxMain++;
  InitControl(btnMNNew, vlyMenu, idxMenu, _M("New"));
      Connect(btnMNNew, CDP_SLOT(CPage_Main, _onMenuFileNew));
      btnMNNew.SetTooltip(_M("Create a new room"));
      idxMenu++;
  InitControl(btnMNOpen, vlyMenu, idxMenu, _M("Open"));
      btnMNOpen.SetTooltip(_M("Open an existing room"));
      idxMenu++;
  InitControl(btnMNSave, vlyMenu, idxMenu, _M("Save"));
      btnMNSave.SetTooltip(_M("Save the current room"));
      idxMenu++;
  InitControl(btnMNSaveAs, vlyMenu, idxMenu, _M("Save As"));
      idxMenu++;
  InitControl(btnMNAbout, vlyMenu, idxMenu, _M("About"), false, false);
      Connect(btnMNAbout, CDP_SLOT(CPage_Main, _onMenuFileAbout));
      btnMNAbout.SetTooltip(_M("About \"VortexGE Room Editor\""));

  // Main work area section
  const uint32_t TOOL_BOX_HEIGHT = 180;
  const uint32_t CAMOV_SEC_WIDTH = 260;
  const uint32_t FOBJ_LIST_WIDTH = 228;
  InitControl(hlyMWork, hlyMain, idxMain, 0, 0);
      hlyMWork.AddBlocksEx(2, 0, TOOL_BOX_HEIGHT);
      idxMain++;
  InitControl(vlyRnLs, hlyMWork, 0, 0, 0);
      if(_iVideoEnv->WindowWidth() > _iVideoEnv->WindowHeight()) {
        const uint32_t ch = MENU_STATUS_BAR_HEIGHT*2 + TOOL_BOX_HEIGHT + 24;
        const uint32_t cw = _iVideoEnv->WindowWidth() - CAMOV_SEC_WIDTH;
        const uint32_t w1 = _iVideoEnv->WindowWidth() - (_iVideoEnv->WindowHeight() - ch) * 5/3;
        const uint32_t w2 = min(w1, cw);
        vlyRnLs.AddBlocksEx(2, 0, max<uint32_t>(w2, FOBJ_LIST_WIDTH));
      }
      else
        vlyRnLs.AddBlocksEx(2, 0, FOBJ_LIST_WIDTH);
  InitControl(vlyOper, hlyMWork, 1, 0, 0);
      vlyOper.AddBlocksEx(2, CAMOV_SEC_WIDTH, 0);

  // Rendering section
  InitControl(pbrRender, vlyRnLs, 0, _M("Preview"));

  // Object list section
  InitControl(pbrList, vlyRnLs, 1, _M("Objects"));
  InitControl(hlyList, &pbrList);
      hlyList.AddBlocksEx(3, 15, 0, 25);
  InitControl(lblList, hlyList, 0, _M("All planes:"));
  InitControl(lstObjects, hlyList, 1);
      Connect(lstObjects, CDP_SLOT(CPage_Main, _onLstObjects));
      lstObjects.SetTooltip(_M("Click to select an object"));
  InitControl(vlyObjectsBtn, hlyList, 2, 0);
      vlyObjectsBtn.AddBlocks(7);
  InitControl(btnEditObject, vlyObjectsBtn, 0, btnEditObject_xpm);
      Connect(btnEditObject, CDP_SLOT(CPage_Main, _onBtnEditObject));
      btnEditObject.SetTooltip(_M("Edit the selected object"));
  InitControl(btnRenObject, vlyObjectsBtn, 1, btnRenObject_xpm);
      Connect(btnRenObject, CDP_SLOT(CPage_Main, _onBtnRenObject));
      btnRenObject.SetTooltip(_M("Rename the selected object"));
  InitControl(btnAlignObject, vlyObjectsBtn, 2, btnAlignObject_xpm);
      Connect(btnAlignObject, CDP_SLOT(CPage_Main, _onBtnAlignObject));
      btnAlignObject.SetTooltip(_M("Align the selected object to another object (selectable)"));
  InitControl(btnCloneObject, vlyObjectsBtn, 3, btnCloneObject_xpm);
      Connect(btnCloneObject, CDP_SLOT(CPage_Main, _onBtnCloneObject));
      btnCloneObject.SetTooltip(_M("Clone the selected object"));
  InitControl(btnDelObject, vlyObjectsBtn, 6, btnDelObject_xpm);
      Connect(btnDelObject, CDP_SLOT(CPage_Main, _onBtnDelObject));
      btnDelObject.SetTooltip(_M("Delete the selected object"));

  // Operations section - camera and movement
  InitControl(hlyCamMovInfo, vlyOper, 0, 0, 0);
      hlyCamMovInfo.AddBlocksEx(2, 0, 75);
  InitControl(vlyCamMovInfo, hlyCamMovInfo, 0, 0, 0);
      vlyCamMovInfo.AddBlocks(2);
  InitControl(pnlCamMovInfo, hlyCamMovInfo, 1);
  InitCompound(hvgCamMovInfo, &pnlCamMovInfo);
      hvgCamMovInfo.AddRows(4);
      for(size_t i = 0; i < 4; ++i) hvgCamMovInfo[i].AddBlocksEx(3, 38, 6, 0);
  InitControl(lblGridSz[0], hvgCamMovInfo[0], 0, _M("Grid"));
  InitControl(lblGridSz[1], hvgCamMovInfo[0], 1, ":");
  InitControl(lblGridSz[2], hvgCamMovInfo[0], 2, "");
  InitControl(lblCurPos[0], hvgCamMovInfo[1], 0, _M("Cursor"));
  InitControl(lblCurPos[1], hvgCamMovInfo[1], 1, ":");
  InitControl(lblCurPos[2], hvgCamMovInfo[1], 2, "");
  InitControl(lblCamPos[0], hvgCamMovInfo[2], 0, _M("Camera"));
  InitControl(lblCamPos[1], hvgCamMovInfo[2], 1, ":");
  InitControl(lblCamPos[2], hvgCamMovInfo[2], 2, "");
  InitControl(lblCamTrg   , hvgCamMovInfo[3], 2, "");

  // Operations section - camera mode
  InitControl(pbrCamMode, vlyCamMovInfo, 0, _M("Camera Mode"));
  InitControl(hlyCamMode, &pbrCamMode);
      hlyCamMode.AddBlocks(4);
  InitControl(rdoCamFollow, hlyCamMode, 0, _M("&Follow"));
      Connect(rdoCamFollow, CDP_SLOT(CPage_Main, _onRdoCamFollowFree));
      rdoCamFollow.SetTooltip(_M("Select follow-camera"));
  InitControl(rdoCamFree, hlyCamMode, 1, _M("F&ree"));
      Connect(rdoCamFree, CDP_SLOT(CPage_Main, _onRdoCamFollowFree));
      rdoCamFree.SetTooltip(_M("Select free-camera"));
  GroupRadioButtons(&rdoCamFollow, &rdoCamFree, NULL);
  InitControl(chkCamFreePos, hlyCamMode, 2, 15, _M("Move &position"));
      chkCamFreePos.SetTooltip(_M("If checked, the free-camera position is moveable"));
  InitControl(chkCamFreeTrg, hlyCamMode, 3, 15, _M("Move &target"));
      Connect(chkCamFreeTrg, CDP_SLOT(CPage_Main, _onChkCamFreeTrg));
      chkCamFreeTrg.SetTooltip(_M("If checked, the free-camera target position is moveable"));

  // Operations section - movement mode
  InitControl(pbrMovMode, vlyCamMovInfo, 1, _M("Movement Mode"));
  InitControl(hlyMovMode, &pbrMovMode);
      hlyMovMode.AddBlocks(4);
  InitControl(rdoMovCurObj, hlyMovMode, 0, _M("Cursor \\& ob&ject"));
      Connect(rdoMovCurObj, CDP_SLOT(CPage_Main, _onRdoMovCurObj));
      rdoMovCurObj.SetTooltip(_M("Select to move the cursor and the selected object"));
  InitControl(rdoMovCurOnly, hlyMovMode, 1, _M("&Cursor only"));
      Connect(rdoMovCurOnly, CDP_SLOT(CPage_Main, _onRdoMovCurCam));
      rdoMovCurOnly.SetTooltip(_M("Select to move the cursor only"));
  InitControl(rdoMovCam, hlyMovMode, 2, _M("Ca&mera"));
      Connect(rdoMovCam, CDP_SLOT(CPage_Main, _onRdoMovCurCam));
      rdoMovCam.SetTooltip(_M("Select to move the camera"));
  GroupRadioButtons(&rdoMovCurObj, &rdoMovCurOnly, &rdoMovCam, NULL);

  // Operations section - objects (main)
  InitControl(pbrObjects, vlyOper, 1, "Objects Management");
  InitCompound(vhgObjects, &pbrObjects);
      const uint32_t cz = 6;
      const uint32_t rz = 5;
      vhgObjects.AddCols(cz, 25);
      vhgObjects.AddColsEx(3, 0, 2, 25);
      for(uint32_t i = 0; i < cz; ++ i) vhgObjects[i].AddBlocks(rz, 25);
      vhgObjects[cz+2].AddBlocks(rz);
  InitControl(vlnObjects, vhgObjects[cz+1], CGLayout::WHOLE_AREA, EBS_Sunken);
  InitControl(btnGrid, vhgObjects[cz+2], 0, btnGrid_xpm);
      Connect(btnGrid, CDP_SLOT(CPage_Main, _onBtnGrid));
      btnGrid.SetTooltip(_M("Change the grid size"));
  InitControl(btnHelp, vhgObjects[cz+2], rz-1, btnHelp_xpm);
      Connect(btnHelp, CDP_SLOT(CPage_Main, _onBtnHelp));
      btnHelp.SetTooltip(_M("Show the \"Quick Reference Page\""));

  // Operations section - objects (plane)
  InitControl(btnNewPlane, vhgObjects[0], 0, btnNewPlane_xpm);
      Connect(btnNewPlane, CDP_SLOT(CPage_Main, _onBtnNewPlane));
      btnNewPlane.SetTooltip(_M("Create a new plane"));

  // Operations section - objects (camera)
  const char **cmXPM[] = { btnFrontCamera_xpm, btnLeftCamera_xpm, btnRightCamera_xpm, btnBackCamera_xpm };
  const char  *cmTTS[] = { "Move camera to the front of the cursor",
                           "Move camera to the left of the cursor",
                           "Move camera to the right of the cursor",
                           "Move camera to the back of the cursor" };
  for(size_t i = 0; i < elemsize(btnResetCamPos); ++ i) {
    InitControl(btnResetCamPos[i], vhgObjects[i], 4, cmXPM[i]);
        Connect(btnResetCamPos[i], CDP_SLOT(CPage_Main, _onBtnResetCamPos));
        btnResetCamPos[i].SetTooltip(_M(cmTTS[i]));
  }

  // Status bar section
  InitControl(stbSB, hlyMain, idxMain, "");

  // Initialize 3D manually and create all the editor's informative objects
  _Initialize3D();
  _Generate3DCursor();
  _Generate3DCameraTarget();
  _Generate3DGrid();

  _iVideoEnv->UseDefaultArrowCursor();
}

void CPage_Main::_Initialize3D()
{
  int32_t px, py, pw, ph, rbsX, rbsY;
  pbrRender.GetClientGeometry(px, py, pw, ph);
  
  const int32_t sw = pw;
  const int32_t sh = ph;
  rbsX = sw;
  rbsY = sw * 3 / 5;
  if(rbsY > sh) {
    rbsX = sh * 5 / 3;
    rbsY = sh;
  }
  rbsX &= 0xFFFFFFF8;
  rbsY &= 0xFFFFFFF8;

  _iVideoEnv->CreateRenderImage(rbsX, rbsY);
  if(lowRes3D)
    _iVideoEnv->CreateRenderBuffer(rbsX/2, rbsY/2);
  else if(rbsX>320 || rbsY>192)
    _iVideoEnv->CreateRenderBuffer(rbsX/2, rbsY/2);
  else
    _iVideoEnv->CreateRenderBuffer(rbsX, rbsY);

  rbpX = px + (pw - rbsX) / 2;
  rbpY = py + (ph - rbsY) / 2;
  _i3DEnv->Initialize(_iVideoEnv);
  _i3DEnv->SetViewport(0, 0, _i3DEnv->RenderBufferWidth(), _i3DEnv->RenderBufferHeight());
  _i3DEnv->SetZClipping(50, 5000);
  _i3DEnv->SetDefaultOmniLightPosition(4000, 8000, -4000);
  _i3DEnv->SetDefaultOmniLightColor(20, 20, 20);
}

void CPage_Main::_Generate3DCursor()
{
  CPlane3DBuilder cbd(TFloatXYZ(5, 5, 5));
#if 0
  cbd.GeneratePlane("x-top",    TFloatXYZ(  0,   5,   5), 1, 5, EPD_0P0);
  cbd.GeneratePlane("x-bottom", TFloatXYZ(  0,   0,   0), 1, 5, EPD_0N0);
  cbd.GeneratePlane("x-left",   TFloatXYZ(  0,   5,   5), 1, 1, EPD_N00);
  cbd.GeneratePlane("x-right",  TFloatXYZ( 25,   5,   0), 1, 1, EPD_P00);
  cbd.GeneratePlane("x-front",  TFloatXYZ(  0,   5,   0), 1, 5, EPD_00N);
  cbd.GeneratePlane("x-back",   TFloatXYZ( 25,   5,   5), 1, 5, EPD_00P);
  cbd.GeneratePlane("y-top",    TFloatXYZ( 10,  15,   5), 1, 1, EPD_0P0);
  cbd.GeneratePlane("y-bottom", TFloatXYZ( 10, -10,   0), 1, 1, EPD_0N0);
  cbd.GeneratePlane("y-left",   TFloatXYZ( 10,  15,   5), 5, 1, EPD_N00);
  cbd.GeneratePlane("y-right",  TFloatXYZ( 15,  15,   0), 5, 1, EPD_P00);
  cbd.GeneratePlane("y-front",  TFloatXYZ( 10,  15,   0), 5, 1, EPD_00N);
  cbd.GeneratePlane("y-back",   TFloatXYZ( 15,  15,   5), 5, 1, EPD_00P);
  cbd.GeneratePlane("z-top",    TFloatXYZ( 10,   5,  15), 5, 1, EPD_0P0);
  cbd.GeneratePlane("z-bottom", TFloatXYZ( 10,   0, -10), 5, 1, EPD_0N0);
  cbd.GeneratePlane("z-left",   TFloatXYZ( 10,   5,  15), 1, 5, EPD_N00);
  cbd.GeneratePlane("z-right",  TFloatXYZ( 15,   5, -10), 1, 5, EPD_P00);
  cbd.GeneratePlane("z-front",  TFloatXYZ( 10,   5, -10), 1, 1, EPD_00N);
  cbd.GeneratePlane("z-back",   TFloatXYZ( 15,   5,  15), 1, 1, EPD_00P);*/
#else
  cbd.SetGridSize(TFloatXYZ(25, 5, 5));
  cbd.GeneratePlane("x-top",    TFloatXYZ(  0,   5,   5), 1, 1, EPD_0P0);
  cbd.GeneratePlane("x-bottom", TFloatXYZ(  0,   0,   0), 1, 1, EPD_0N0);
  cbd.GeneratePlane("x-front",  TFloatXYZ(  0,   5,   0), 1, 1, EPD_00N);
  cbd.GeneratePlane("x-back",   TFloatXYZ( 25,   5,   5), 1, 1, EPD_00P);
  cbd.SetGridSize(TFloatXYZ(5, 25, 5));
  cbd.GeneratePlane("y-left",   TFloatXYZ( 10,  15,   5), 1, 1, EPD_N00);
  cbd.GeneratePlane("y-right",  TFloatXYZ( 15,  15,   0), 1, 1, EPD_P00);
  cbd.GeneratePlane("y-front",  TFloatXYZ( 10,  15,   0), 1, 1, EPD_00N);
  cbd.GeneratePlane("y-back",   TFloatXYZ( 15,  15,   5), 1, 1, EPD_00P);
  cbd.SetGridSize(TFloatXYZ(5, 5, 25));
  cbd.GeneratePlane("z-top",    TFloatXYZ( 10,   5,  15), 1, 1, EPD_0P0);
  cbd.GeneratePlane("z-bottom", TFloatXYZ( 10,   0, -10), 1, 1, EPD_0N0);
  cbd.GeneratePlane("z-left",   TFloatXYZ( 10,   5,  15), 1, 1, EPD_N00);
  cbd.GeneratePlane("z-right",  TFloatXYZ( 15,   5, -10), 1, 1, EPD_P00);
  cbd.SetGridSize(TFloatXYZ(5, 5, 5));
  cbd.GeneratePlane("x-left",   TFloatXYZ(  0,   5,   5), 1, 1, EPD_N00);
  cbd.GeneratePlane("x-right",  TFloatXYZ( 25,   5,   0), 1, 1, EPD_P00);
  cbd.GeneratePlane("y-top",    TFloatXYZ( 10,  15,   5), 1, 1, EPD_0P0);
  cbd.GeneratePlane("y-bottom", TFloatXYZ( 10, -10,   0), 1, 1, EPD_0N0);
  cbd.GeneratePlane("z-front",  TFloatXYZ( 10,   5, -10), 1, 1, EPD_00N);
  cbd.GeneratePlane("z-back",   TFloatXYZ( 15,   5,  15), 1, 1, EPD_00P);
#endif
  cbd.GeneratePanel3DBuffer(_cursor);
  _cursor.LocalRegionTranslateAllBalanced();
  _cursor.LocalXYZScale(0.5f, 0.5f, 0.5f);
  _cursor.SetColor(31, 31, 0);
}

void CPage_Main::_Generate3DCameraTarget()
{
  const float    ctV[] = { 5,10,5,  0,0,10,  10,0,10,  0,0,0,  10,0,0 };
  const uint32_t ctF[] = { 1,0,0,3,  2,0,0,1,  4,0,0,2,  3,0,0,4,  1,3,4,2 };
  _camTarget.InitFromMemory(ctV, 5, ctF, 5);
  _camTarget.LocalRegionTranslateAllBalanced();
  _camTarget.LocalXYZScale(0.5f, 0.5f, 0.5f);
  _camTarget.LocalRegionTranslateCenterOfWeight();
  _camTarget.SetColor(0, 31, 0);
}

void CPage_Main::_Generate3DGrid()
{
  // Generate texture
  CImageBuilder ib;
  ib.CreateEmptyImage(128, 128);
  ib.ClearImage(0, 0, 15);
  TUN8RGB *lines[] = { ib.GetLineBuffer(0), ib.GetLineBuffer(1),
                       ib.GetLineBuffer(2), ib.GetLineBuffer(3),
                       ib.GetLineBuffer(ib.GetHeight() - 4),
                       ib.GetLineBuffer(ib.GetHeight() - 3),
                       ib.GetLineBuffer(ib.GetHeight() - 2),
                       ib.GetLineBuffer(ib.GetHeight() - 1) };
  for(uint32_t c = 0; c < elemsize(lines); ++c) {
    for(uint32_t x = 0; x < ib.GetWidth(); ++x) (lines[c] + x)->un8B = 31;
  }
  const uint32_t ofs[] = { 0, 1, 2, 3,
                          ib.GetWidth() - 4, ib.GetWidth() - 3,
                          ib.GetWidth() - 2, ib.GetWidth() - 1 };
  for(uint32_t y = 4; y < (ib.GetHeight() - 4); ++y) {
    TUN8RGB *line = ib.GetLineBuffer(y);
    for(uint32_t c = 0; c < elemsize(ofs); ++c) (line + ofs[c])->un8B = 31;
  }
  ib.BlurEffect(2);
  ib.TransferToTexture(_gridTex);

  // Generate panels
  const TFloatXYZ &gz = _roomBuilder.GetRoomGridSize();
  const uint32_t   gs = 4;
  CPlane3DBuilder  cbd(gz);
  cbd.GeneratePlane("grid-p", TFloatXYZ(0, 0, 0), gs, gs, EPD_0P0);
  cbd.GeneratePlane("grid-n", TFloatXYZ(0, 0, -gz.z*gs), gs, gs, EPD_0N0);
  cbd.GeneratePanel3DBuffer(_grid, &_gridTex);
  _grid.LocalRegionTranslateAllBalanced();
}

void CPage_Main::_SelectDefCamPos(uint32_t idx)
{
  const TFloatXYZ &gz = _roomBuilder.GetRoomGridSize();

       if(idx == 3) _loc.lCamOffset = TFloatXYZ(    0, gz.y,  gz.z); // Back
  else if(idx == 2) _loc.lCamOffset = TFloatXYZ( gz.x, gz.y,     0); // Right
  else if(idx == 1) _loc.lCamOffset = TFloatXYZ(-gz.x, gz.y,     0); // Left
  else              _loc.lCamOffset = TFloatXYZ(    0, gz.y, -gz.z); // Front

  _loc.fCamTrg = _loc.curPos;
  _loc.fCamPos = _loc.curPos + _loc.lCamOffset;
}
