// ############################################################################################
//  roomedit-DlgPages-Main.h
//  Definition & implementation of roomedit's CPage_Main class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _UTIL___ROOMEDIT_DLGPAGES_MAIN_H_
#define _UTIL___ROOMEDIT_DLGPAGES_MAIN_H_

// roomedit's headers
#include "roomedit-RoomBuilder.h"


// Foward reference of the application class
class CRoomEdit;

// Foward reference of the structure used for storing plane settings
struct TInit_PlaneSettings;


//
// Our dialog page: Main Page
//
class CPage_Main : public CDialogPage {
  public:
    // Key map for movements
    struct _TKeyMap {
      bool ctrl, alt, shift;
      bool up, down, left, right;
      bool home, pgup;

      inline _TKeyMap()
      : ctrl(false), alt (false), shift(false),
        up  (false), down(false), left (false), right(false),
        home(false), pgup(false)
      {}
    } _keyMap;

  public:
    CPage_Main(CRoomEdit &appClass, bool lowRes3D_);
    virtual ~CPage_Main() throw();

    // Information functions
    inline CRoomEdit &GetAppInst()
    { return(_app); }
    inline bool PlaneIsExist(const string &planeName) const
    { return(lstObjects.ItemIsExist(planeName)); }

    // Keyboard shortcuts' event handlers
    void _onKscCamFollow();
    void _onKscCamFree();
    void _onKscCamMovPos();
    void _onKscCamMovTrg();
    void _onKscMovCurObj();
    void _onKscMovCurOnly();
    void _onKscMovCam();
    // Simply delegate these keyboard shortcuts' event handlers to the
    // corresponding controls' event handlers
    CDP_KSC_EVENT_DELEGATE(_onKscMovCam2F,  btnResetCamPos[0],  _onBtnResetCamPos)
    CDP_KSC_EVENT_DELEGATE(_onKscMovCam2L,  btnResetCamPos[1],  _onBtnResetCamPos)
    CDP_KSC_EVENT_DELEGATE(_onKscMovCam2R,  btnResetCamPos[2],  _onBtnResetCamPos)
    CDP_KSC_EVENT_DELEGATE(_onKscMovCam2B,  btnResetCamPos[3],  _onBtnResetCamPos)
    CDP_KSC_EVENT_DELEGATE(_onKscEditObj,   btnEditObject,      _onBtnEditObject )
    CDP_KSC_EVENT_DELEGATE(_onKscRenObj,    btnRenObject,       _onBtnRenObject  )
    CDP_KSC_EVENT_DELEGATE(_onKscAlignObj,  btnAlignObject,     _onBtnAlignObject)
    CDP_KSC_EVENT_DELEGATE(_onKscCloneObj,  btnCloneObject,     _onBtnCloneObject)
    CDP_KSC_EVENT_DELEGATE(_onKscDelObj,    btnDelObject,       _onBtnDelObject  )
    CDP_KSC_EVENT_DELEGATE(_onKscNewPlane,  btnNewPlane,        _onBtnNewPlane   )
    CDP_KSC_EVENT_DELEGATE(_onKscGrid,      btnGrid,            _onBtnGrid       )
    CDP_KSC_EVENT_DELEGATE(_onKscHelp,      btnHelp,            _onBtnHelp       )

    // Controls' event handlers: Menu bar section
    void _onMenuFileNew(_CControl *ctrl);
    void _onMenuFileNew(const TFloatXYZ &gridSize);
    void _onMenuFileNew(const string &fileName, const TFloatXYZ &gridSize);
    void _onMenuFileAbout(_CControl *ctrl);
    // Controls' event handlers: Object list section
    void _onLstObjects(_CControl *ctrl, bool doubleClick);
    void _onBtnEditObject(_CControl *ctrl);
    void _onBtnRenObject(_CControl *ctrl);
    void _onBtnAlignObject(_CControl *ctrl);
    void _onBtnCloneObject(_CControl *ctrl);
    void _onBtnDelObject(_CControl *ctrl);
    // Controls' event handlers: Operations section - camera mode
    void _onRdoCamFollowFree(_CControl *ctrl);
    void _onChkCamFreeTrg(_CControl *ctrl);
    // Controls' event handlers: Operations section - movement mode
    void _onRdoMovCurObj(_CControl *ctrl);
    void _onRdoMovCurCam(_CControl *ctrl);
    // Controls' event handlers: Operations section - objects (main)
    void _onBtnGrid(_CControl *ctrl);
    void _onBtnGrid(const TFloatXYZ &gridSize);
    void _onBtnHelp(_CControl *ctrl);
    // Controls' event handlers: Operations section - objects (plane)
    void _onBtnNewPlane(_CControl *ctrl);
    void _onBtnNewPlane(const TInit_PlaneSettings &init);
    void _onRdoMovCurObj_Plane();
    void _onBtnEditObject_Plane();
    void _onBtnEditObject_Plane(const TInit_PlaneSettings &init);
    void _onBtnRenObject_Plane();
    void _onBtnDelObject_Plane();
    void _onMove_Plane(const TFloatXYZ &mf);
    // Controls: Operations section - objects (camera)
    void _onBtnResetCamPos(_CControl *ctrl);

    // Management functions
    void ResetAllControls();
    void UpdateAllControlsEnableStates();
    void UpdateInfoText();
    void HandleMovements();

    // Rendering functions
    void MarkAsDirty();
    void UpdateCameraPos();
    void RenderRoom(bool updateFrameInfo);
    void SmartUpdateAll();

    // Reimplemented from CDialogPage
    virtual void HandleEvent_OnKeyPress(EKeySymbol keySymbol, const char *str);
    virtual void HandleEvent_OnKeyRelease(EKeySymbol keySymbol, const char *str);
    virtual void HandleEvent_OnUpdateFrame(bool windowIsActive);

    // Reimplemented from CDialogPage
    virtual void InitializeAllControls();

  protected:
    // Constants for rendering & updating
    static const uint32_t DIRTY_MAX = 5;

    // Constants for object selection
    static const uint32_t OBJECT_TAB_PLANE = 0;
    static const uint32_t OBJECT_TAB_MAX   = 0;

    // Reimplemented from CDialogPage
    virtual void DisplayTooltip(const char *tooltip);
    virtual void CustomDraw_OnRedrawAllControls();
    virtual void CustomDraw_OnButtonPress(EBtnSymbol eButtonSym);
    virtual void CustomDraw_OnButtonRelease(EBtnSymbol eButtonSym);
    virtual void CustomDraw_OnMouseMove(int32_t pointerXPos, int32_t pointerYPos);

    // Helper functions
    void _Initialize3D();
    void _Generate3DCursor();
    void _Generate3DCameraTarget();
    void _Generate3DGrid();
    void _SelectDefCamPos(uint32_t idx);

    // Main data
    CRoomEdit     &_app;
    CVideoStorage  _vStrg;
    uint32_t       _activeObjectTab;
    // Room's editor data
    string         _roomFileName;
    CRoomBuilder   _roomBuilder;
    uint32_t       _dirty;
    struct {
      TFloatXYZ curPos; // Current cursor position
      TFloatXYZ camPos; // Current camera position
      TFloatXYZ camTrg; // Current camera target
      CMatrix3D matCam; // Current camera matrix

      TFloatXYZ lCamOffset; // Offset for follow-camera
      TFloatXYZ fCamPos;    // Position for free-camera
      TFloatXYZ fCamTrg;    // Target for free-camera
    } _loc;
    // Informative objects
    CTexture       _gridTex;
    CPanel3DBuffer _grid;
    CPanel3DBuffer _cursor;
    CPanel3DBuffer _camTarget;

    // Controls: Main section
    CHLayout          hlyMain;
    // Controls: Menu bar section
    CVLayout          vlyMenu;
    CButton           btnMNNew;
    CButton           btnMNOpen;
    CButton           btnMNSave;
    CButton           btnMNSaveAs;
    CButton           btnMNAbout;
    // Controls: Main work area section
    CHLayout          hlyMWork;
    CVLayout          vlyRnLs;
    CVLayout          vlyOper;
    // Controls: Rendering section
    CPanelBar         pbrRender;
    uint32_t          rbpX, rbpY;
    bool              lowRes3D;
    // Controls: Object list section
    CPanelBar         pbrList;
    CHLayout          hlyList;
    CLabel            lblList;
    CListBox          lstObjects;
    CVLayout          vlyObjectsBtn;
    CButton           btnEditObject;
    CButton           btnRenObject;
    CButton           btnAlignObject;
    CButton           btnCloneObject;
    CButton           btnDelObject;
    // Controls: Operations section - camera, movement & information
    CHLayout          hlyCamMovInfo;
    CVLayout          vlyCamMovInfo;
    CPanel            pnlCamMovInfo;
    CHVLayoutCompound hvgCamMovInfo;
    CLabel            lblGridSz[3];
    CLabel            lblCurPos[3];
    CLabel            lblCamPos[3];
    CLabel            lblCamTrg;
    // Controls: Operations section - camera mode
    CPanelBar         pbrCamMode;
    CHLayout          hlyCamMode;
    CRadioButton      rdoCamFollow;
    CRadioButton      rdoCamFree;
    CCheckBox         chkCamFreePos;
    CCheckBox         chkCamFreeTrg;
    // Controls: Operations section - movement mode
    CPanelBar         pbrMovMode;
    CHLayout          hlyMovMode;
    CRadioButton      rdoMovCurObj;
    CRadioButton      rdoMovCurOnly;
    CRadioButton      rdoMovCam;
    // Controls: Operations section - objects (main)
    CPanelBar         pbrObjects;
    CVHLayoutCompound vhgObjects;
    CVLine            vlnObjects;
    CButton           btnGrid;
    CButton           btnHelp;
    // Controls: Operations section - objects (plane)
    CButton           btnNewPlane;
    // Controls: Operations section - objects (camera)
    CButton           btnResetCamPos[4];
    // Controls: Status bar section
    CStatusBar        stbSB;
};

#endif
