// ############################################################################################
//  roomedit-DlgPages-QRP.h
//  Implementation of roomedit's CPage_QRP class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// roomedit's headers
#include "roomedit-DlgPages-QRP.h"
#include "roomedit-RoomEdit.h"


//
// Help text
//
//! \todo Don't forget to translate them too later ;)
//! \todo Find better way than just writing them in this file :P
static const char *_helpTopic[] = { "Generic informations",
                                    "Keyboard shortcuts usable in the main editing page",
                                    "Cursor, objects and camera movements" };
                                  
static const char *_helpText[]  = {
  "There are 3 helper objects in the \"Preview\" panel-bar:\n" \
  "  1. The cursor (the plus-shaped object).\n" \
  "  2. The free-camera's target (the triangle-shaped object).\n" \
  "  3. The blue XZ-plane grid, always attached to the cursor."
  ,
  "\"Camera Mode\" selection:\n" \
  "   1. 'f' : Select follow-camera.\n" \
  "   2. 'r' : Select free-camera.\n" \
  "   3. 'p' : Check/uncheck the \"Camera Mode/Move position\" check-box.\n" \
  "   4. 't' : Check/uncheck the \"Camera Mode/Move target\" check-box.\n" \
  "\n" \
  "\"Movement Mode\" selection:\n" \
  "  1. 'j' : Select to move the cursor & the selected object.\n" \
  "  2. 'c' : Select to move the cursor only.\n" \
  "  3. 'm' : Select to move the camera.\n" \
  "\n" \
  "Resetting follow-camera's and free-camera's positions:\n" \
  "  1. shift + 'f' : Move camera to the front of the cursor.\n" \
  "  2. shift + 'l' : Move camera to the left of the cursor.\n" \
  "  3. shift + 'r' : Move camera to the right of the cursor.\n" \
  "  4. shift + 'b' : Move camera to the back of the cursor.\n" \
  "\n" \
  "Common object operations:\n" \
  "  1. ctrl - 'e' : Edit the selected object.\n" \
  "  2. ctrl - 'r' : Rename the selected object.\n" \
  "  3. ctrl - 'a' : Align the selected object to another object (selectable).\n" \
  "  4. ctrl - 'c' : Clone the selected object.\n" \
  "  5.        del : Delete the selected object.\n"
  "\n" \
  "Specific object operations:\n" \
  "  1. ctrl - shift - 'p' : Create a new plane.\n" \
  "\n" \
  "Other shortcuts:\n" \
  "  1. ctrl - 'g' : Change the grid size.\n" \
  "  2.        F10 : Show this \"Quick Reference Page\".\n" \
  ,
  "Movement keys (for moving cursor, objects and camera :\n" \
  "  * Use up, down, left and right arrows to move in the horizontal direction (in the XZ-plane).\n" \
  "  * Use home and page-up to move in the vertical direction (in the Y-axis).\n" \
  "  * Hold control when moving to perform single-step movement.\n" \
  "\n" \
  "Movement modes:\n" \
  "  1. Moving the cursor and the selected object. In this mode, the cursor will be rendered in red.\n" \
  "  2. Moving the cursor only. In this mode, the cursor will be rendered in yellow.\n" \
  "  3. Moving the camera. In this mode, the cursor will be rendered in gray.\n" \
  "     a. In follow-camera mode, only the camera's position is moveable (the camera's target is attached to the cursor).\n" \
  "     b. In free-camera mode, both the camera's position and target can be moveable (depends on the check boxes' values\n" \
  "        in the \"Camera Mode\" panel-bar). If the camera's target is moveable it will be rendered in green, otherwise\n" \
  "        it will be rendered in gray."
};


//
// Our dialog page: quick reference page
//
CPage_QRP::CPage_QRP(CPage_Main &mainPage)
: CDialogPage(mainPage.GetAppInst(), &mainPage.GetAppInst()),
  _app       (mainPage.GetAppInst()),
  _mPage     (mainPage)
{
}

CPage_QRP::~CPage_QRP() throw()
{
}

void CPage_QRP::_onLstTopic(_CControl *ctrl, bool doubleClick)
{
  UNUSED_PARAMS_2(ctrl, doubleClick);
  txtHelp.SetText(_helpText[lstTopic.GetSelectedIndex()]);
  txtHelp.DrawControl();
  _iVideoEnv->UpdateDisplay();
}

void CPage_QRP::_onBtnDone(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  _app.SelectPage(CRoomEdit::PAGE_MAIN);
}

void CPage_QRP::InitializeAllControls()
{
  const uint32_t pw = _iVideoEnv->WindowWidth();
  const uint32_t ph = _iVideoEnv->WindowHeight();
  const uint32_t px = 0;
  const uint32_t py = 0;
  const uint32_t th = ph / 5;
  const uint32_t lh = (th < 100) ? 100 : ((th > 150) ? 150: th);

  // Main section
  InitControl(pbrMain, NULL, px, py, pw, ph, _M("Quick Reference Page"));
  InitControl(hlyMain, &pbrMain, CGLayout::DEFAULT_PAD_SIZE, 0);
      hlyMain.AddBlocksEx(7, 20, lh, 10, 20, 0, 5, 30);
  InitControl(lblTopic, hlyMain, 0, _M("Select topic from the list box:"));
  InitControl(lstTopic, hlyMain, 1);
      Connect(lstTopic, CDP_SLOT(CPage_QRP, _onLstTopic));
      for(size_t i = 0; i < elemsize(_helpTopic); ++i) lstTopic.AddItem(_helpTopic[i]);
  InitControl(lblHelp, hlyMain, 3, _M("Click on the read-only text box below to navigate long text:"));
  InitControl(txtHelp, hlyMain, 4);
      txtHelp.SetReadOnly();
      txtHelp.SetText(_helpText[0]);
  InitControl(btnDone, hlyMain, 6, _M("Done"), true, true);
      Connect(btnDone, CDP_SLOT(CPage_QRP, _onBtnDone));
}

void CPage_QRP::HandleEvent_OnKeyPress(EKeySymbol keySymbol, const char *str)
{
  UNUSED_PARAM(str);

  if(keySymbol==VKS_ENTER || keySymbol==VKS_KP_ENTER || keySymbol == VKS_ESC)
    _onBtnDone(NULL);
}
